@@ -6,7 +6,7 @@ from sklearn.linear_model import LinearRegression
 from sklearn.dummy import DummyRegressor
 
 from .double_ml import DoubleML
-from ._utils import _dml_cv_predict, _dml_tune
+from ._utils import _dml_cv_predict, _dml_tune, _check_finite_predictions
 
 
 class DoubleMLPLIV(DoubleML):
@@ -282,17 +282,21 @@ class DoubleMLPLIV(DoubleML):
         return res
 
     def _ml_nuisance_and_score_elements_partial_x(self, smpls, n_jobs_cv):
-        x, y = check_X_y(self._dml_data.x, self._dml_data.y)
-        x, d = check_X_y(x, self._dml_data.d)
+        x, y = check_X_y(self._dml_data.x, self._dml_data.y,
+                         force_all_finite=False)
+        x, d = check_X_y(x, self._dml_data.d,
+                         force_all_finite=False)
 
         # nuisance g
         g_hat = _dml_cv_predict(self._learner['ml_g'], x, y, smpls=smpls, n_jobs=n_jobs_cv,
                                 est_params=self._get_params('ml_g'), method=self._predict_method['ml_g'])
+        _check_finite_predictions(g_hat, self._learner['ml_g'], 'ml_g', smpls)
 
         # nuisance m
         if self._dml_data.n_instr == 1:
             # one instrument: just identified
-            x, z = check_X_y(x, np.ravel(self._dml_data.z))
+            x, z = check_X_y(x, np.ravel(self._dml_data.z),
+                             force_all_finite=False)
             m_hat = _dml_cv_predict(self._learner['ml_m'], x, z, smpls=smpls, n_jobs=n_jobs_cv,
                                     est_params=self._get_params('ml_m'), method=self._predict_method['ml_m'])
         else:
@@ -300,14 +304,17 @@ class DoubleMLPLIV(DoubleML):
             m_hat = np.full((self._dml_data.n_obs, self._dml_data.n_instr), np.nan)
             z = self._dml_data.z
             for i_instr in range(self._dml_data.n_instr):
-                x, this_z = check_X_y(x, z[:, i_instr])
+                x, this_z = check_X_y(x, z[:, i_instr],
+                                      force_all_finite=False)
                 m_hat[:, i_instr] = _dml_cv_predict(self._learner['ml_m'], x, this_z, smpls=smpls, n_jobs=n_jobs_cv,
                                                     est_params=self._get_params('ml_m_' + self._dml_data.z_cols[i_instr]),
                                                     method=self._predict_method['ml_m'])
+        _check_finite_predictions(m_hat, self._learner['ml_m'], 'ml_m', smpls)
 
         # nuisance r
         r_hat = _dml_cv_predict(self._learner['ml_r'], x, d, smpls=smpls, n_jobs=n_jobs_cv,
                                 est_params=self._get_params('ml_r'), method=self._predict_method['ml_r'])
+        _check_finite_predictions(r_hat, self._learner['ml_r'], 'ml_r', smpls)
 
         psi_a, psi_b = self._score_elements(y, z, d, g_hat, m_hat, r_hat, smpls)
         preds = {'ml_g': g_hat,
@@ -353,11 +360,13 @@ class DoubleMLPLIV(DoubleML):
     def _ml_nuisance_and_score_elements_partial_z(self, smpls, n_jobs_cv):
         y = self._dml_data.y
         xz, d = check_X_y(np.hstack((self._dml_data.x, self._dml_data.z)),
-                          self._dml_data.d)
+                          self._dml_data.d,
+                          force_all_finite=False)
 
         # nuisance m
         r_hat = _dml_cv_predict(self._learner['ml_r'], xz, d, smpls=smpls, n_jobs=n_jobs_cv,
                                 est_params=self._get_params('ml_r'), method=self._predict_method['ml_r'])
+        _check_finite_predictions(r_hat, self._learner['ml_r'], 'ml_r', smpls)
 
         if isinstance(self.score, str):
             assert self.score == 'partialling out'
@@ -372,23 +381,29 @@ class DoubleMLPLIV(DoubleML):
         return psi_a, psi_b, preds
 
     def _ml_nuisance_and_score_elements_partial_xz(self, smpls, n_jobs_cv):
-        x, y = check_X_y(self._dml_data.x, self._dml_data.y)
+        x, y = check_X_y(self._dml_data.x, self._dml_data.y,
+                         force_all_finite=False)
         xz, d = check_X_y(np.hstack((self._dml_data.x, self._dml_data.z)),
-                          self._dml_data.d)
-        x, d = check_X_y(x, self._dml_data.d)
+                          self._dml_data.d,
+                          force_all_finite=False)
+        x, d = check_X_y(x, self._dml_data.d,
+                         force_all_finite=False)
 
         # nuisance g
         g_hat = _dml_cv_predict(self._learner['ml_g'], x, y, smpls=smpls, n_jobs=n_jobs_cv,
                                 est_params=self._get_params('ml_g'), method=self._predict_method['ml_g'])
+        _check_finite_predictions(g_hat, self._learner['ml_g'], 'ml_g', smpls)
 
         # nuisance m
         m_hat, m_hat_on_train = _dml_cv_predict(self._learner['ml_m'], xz, d, smpls=smpls, n_jobs=n_jobs_cv,
                                                 est_params=self._get_params('ml_m'), return_train_preds=True,
                                                 method=self._predict_method['ml_m'])
+        _check_finite_predictions(m_hat, self._learner['ml_m'], 'ml_m', smpls)
 
         # nuisance r
         m_hat_tilde = _dml_cv_predict(self._learner['ml_r'], x, m_hat_on_train, smpls=smpls, n_jobs=n_jobs_cv,
                                       est_params=self._get_params('ml_r'), method=self._predict_method['ml_r'])
+        _check_finite_predictions(m_hat_tilde, self._learner['ml_r'], 'ml_r', smpls)
 
         # compute residuals
         u_hat = y - g_hat
@@ -410,8 +425,10 @@ class DoubleMLPLIV(DoubleML):
 
     def _ml_nuisance_tuning_partial_x(self, smpls, param_grids, scoring_methods, n_folds_tune, n_jobs_cv,
                                       search_mode, n_iter_randomized_search):
-        x, y = check_X_y(self._dml_data.x, self._dml_data.y)
-        x, d = check_X_y(x, self._dml_data.d)
+        x, y = check_X_y(self._dml_data.x, self._dml_data.y,
+                         force_all_finite=False)
+        x, d = check_X_y(x, self._dml_data.d,
+                         force_all_finite=False)
 
         if scoring_methods is None:
             scoring_methods = {'ml_g': None,
@@ -428,7 +445,8 @@ class DoubleMLPLIV(DoubleML):
             m_tune_res = {instr_var: list() for instr_var in self._dml_data.z_cols}
             z = self._dml_data.z
             for i_instr in range(self._dml_data.n_instr):
-                x, this_z = check_X_y(x, z[:, i_instr])
+                x, this_z = check_X_y(x, z[:, i_instr],
+                                      force_all_finite=False)
                 m_tune_res[self._dml_data.z_cols[i_instr]] = _dml_tune(this_z, x, train_inds,
                                                                        self._learner['ml_m'], param_grids['ml_m'],
                                                                        scoring_methods['ml_m'],
@@ -436,7 +454,8 @@ class DoubleMLPLIV(DoubleML):
                                                                        n_iter_randomized_search)
         else:
             # one instrument: just identified
-            x, z = check_X_y(x, np.ravel(self._dml_data.z))
+            x, z = check_X_y(x, np.ravel(self._dml_data.z),
+                             force_all_finite=False)
             m_tune_res = _dml_tune(z, x, train_inds,
                                    self._learner['ml_m'], param_grids['ml_m'], scoring_methods['ml_m'],
                                    n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
@@ -470,7 +489,8 @@ class DoubleMLPLIV(DoubleML):
     def _ml_nuisance_tuning_partial_z(self, smpls, param_grids, scoring_methods, n_folds_tune, n_jobs_cv,
                                       search_mode, n_iter_randomized_search):
         xz, d = check_X_y(np.hstack((self._dml_data.x, self._dml_data.z)),
-                          self._dml_data.d)
+                          self._dml_data.d,
+                          force_all_finite=False)
 
         if scoring_methods is None:
             scoring_methods = {'ml_r': None}
@@ -493,10 +513,13 @@ class DoubleMLPLIV(DoubleML):
 
     def _ml_nuisance_tuning_partial_xz(self, smpls, param_grids, scoring_methods, n_folds_tune, n_jobs_cv,
                                        search_mode, n_iter_randomized_search):
-        x, y = check_X_y(self._dml_data.x, self._dml_data.y)
+        x, y = check_X_y(self._dml_data.x, self._dml_data.y,
+                         force_all_finite=False)
         xz, d = check_X_y(np.hstack((self._dml_data.x, self._dml_data.z)),
-                          self._dml_data.d)
-        x, d = check_X_y(x, self._dml_data.d)
+                          self._dml_data.d,
+                          force_all_finite=False)
+        x, d = check_X_y(x, self._dml_data.d,
+                         force_all_finite=False)
 
         if scoring_methods is None:
             scoring_methods = {'ml_g': None,
