@@ -12,7 +12,7 @@ if (on_cran) {
     score = "ATTE",
     trimming_threshold = 0,
     stringsAsFactors = FALSE)
-  test_cases["test_name"] = apply(test_cases, 1, paste, collapse = "_")
+  test_cases[".test_name"] = apply(test_cases, 1, paste, collapse = "_")
 } else {
   test_cases = expand.grid(
     learner = "cv_glmnet",
@@ -20,7 +20,7 @@ if (on_cran) {
     score = c("ATE", "ATTE"),
     trimming_threshold = 0,
     stringsAsFactors = FALSE)
-  test_cases["test_name"] = apply(test_cases, 1, paste, collapse = "_")
+  test_cases[".test_name"] = apply(test_cases, 1, paste, collapse = "_")
 }
 
 patrick::with_parameters_test_that("Unit tests for IRM:",
@@ -36,14 +36,14 @@ patrick::with_parameters_test_that("Unit tests for IRM:",
       dml_procedure = dml_procedure, score = score)
     theta = irm_hat$coef
     se = irm_hat$se
-    
+
     boot_theta = bootstrap_irm(irm_hat$thetas, irm_hat$ses,
-                               data_irm$df,
-                               y = "y", d = "d",
-                               n_folds = 5, smpls = irm_hat$smpls,
-                               all_preds= irm_hat$all_preds,
-                               score = score,
-                               bootstrap = "normal", n_rep_boot = n_rep_boot)$boot_coef
+      data_irm$df,
+      y = "y", d = "d",
+      n_folds = 5, smpls = irm_hat$smpls,
+      all_preds = irm_hat$all_preds,
+      score = score,
+      bootstrap = "normal", n_rep_boot = n_rep_boot)$boot_coef
 
 
     set.seed(3141)
@@ -61,7 +61,7 @@ patrick::with_parameters_test_that("Unit tests for IRM:",
     se_obj = double_mlirm_obj$se
 
     # bootstrap
-    double_mlirm_obj$bootstrap(method = 'normal',  n_rep = n_rep_boot)
+    double_mlirm_obj$bootstrap(method = "normal", n_rep = n_rep_boot)
     boot_theta_obj = double_mlirm_obj$boot_coef
 
     # at the moment the object result comes without a name
