@@ -18,7 +18,7 @@ if (on_cran) {
     score = "partialling out",
     stringsAsFactors = FALSE)
 }
-test_cases["test_name"] = apply(test_cases, 1, paste, collapse = "_")
+test_cases[".test_name"] = apply(test_cases, 1, paste, collapse = "_")
 
 patrick::with_parameters_test_that("Unit tests for PLIV:",
   .cases = test_cases, {
@@ -35,13 +35,13 @@ patrick::with_parameters_test_that("Unit tests for PLIV:",
       dml_procedure = dml_procedure, score = score)
     theta = pliv_hat$coef
     se = pliv_hat$se
-    
+
     boot_theta = bootstrap_pliv(pliv_hat$thetas, pliv_hat$ses,
-                                 data_pliv$df,
-                                 y = "y", d = "d", z = "z",
-                                 n_folds = 5, smpls = pliv_hat$smpls,
-                                 all_preds= pliv_hat$all_preds,
-                                 bootstrap = "normal", n_rep_boot = n_rep_boot)$boot_coef
+      data_pliv$df,
+      y = "y", d = "d", z = "z",
+      n_folds = 5, smpls = pliv_hat$smpls,
+      all_preds = pliv_hat$all_preds,
+      bootstrap = "normal", n_rep_boot = n_rep_boot)$boot_coef
 
     set.seed(3141)
     double_mlpliv_obj = DoubleMLPLIV$new(
@@ -58,7 +58,7 @@ patrick::with_parameters_test_that("Unit tests for PLIV:",
     se_obj = double_mlpliv_obj$se
 
     # bootstrap
-    double_mlpliv_obj$bootstrap(method = 'normal',  n_rep = n_rep_boot)
+    double_mlpliv_obj$bootstrap(method = "normal", n_rep = n_rep_boot)
     boot_theta_obj = double_mlpliv_obj$boot_coef
 
     # at the moment the object result comes without a name
