@@ -10,7 +10,7 @@ test_cases = expand.grid(
   dml_procedure = "dml2",
   score = "partialling out",
   stringsAsFactors = FALSE)
-test_cases["test_name"] = apply(test_cases, 1, paste, collapse = "_")
+test_cases[".test_name"] = apply(test_cases, 1, paste, collapse = "_")
 
 patrick::with_parameters_test_that("Unit tests for PLR with classifier for ml_m:",
   .cases = test_cases, {
