@@ -22,13 +22,13 @@ if (on_cran) {
     n_folds = c(1, 2),
     stringsAsFactors = FALSE)
 }
-test_cases["test_name"] = apply(test_cases, 1, paste, collapse = "_")
+test_cases[".test_name"] = apply(test_cases, 1, paste, collapse = "_")
 
 patrick::with_parameters_test_that("Unit tests for PLR:",
   .cases = test_cases, {
     learner_pars = get_default_mlmethod_plr(learner)
     n_rep_boot = 498
-    
+
     set.seed(3141)
     df = data_plr$df
     if (n_folds == 2) {
@@ -36,19 +36,20 @@ patrick::with_parameters_test_that("Unit tests for PLR:",
       my_sampling = rsmp("holdout", ratio = 0.5)$instantiate(my_task)
       train_ids = list(my_sampling$train_set(1))
       test_ids = list(my_sampling$test_set(1))
-      
+
       smpls = list(list(train_ids = train_ids, test_ids = test_ids))
     } else {
-      smpls = list(list(train_ids = list(seq(nrow(df))),
-                        test_ids = list(seq(nrow(df)))))
+      smpls = list(list(
+        train_ids = list(seq(nrow(df))),
+        test_ids = list(seq(nrow(df)))))
     }
     plr_hat = dml_plr(df,
-                      y = "y", d = "d",
-                      n_folds = 1,
-                      ml_g = learner_pars$ml_g$clone(),
-                      ml_m = learner_pars$ml_m$clone(),
-                      dml_procedure = dml_procedure, score = score,
-                      smpls=smpls)
+      y = "y", d = "d",
+      n_folds = 1,
+      ml_g = learner_pars$ml_g$clone(),
+      ml_m = learner_pars$ml_m$clone(),
+      dml_procedure = dml_procedure, score = score,
+      smpls = smpls)
     theta = plr_hat$coef
     se = plr_hat$se
     t = plr_hat$t
@@ -64,7 +65,7 @@ patrick::with_parameters_test_that("Unit tests for PLR:",
       score = score,
       apply_cross_fitting = apply_cross_fitting)
 
-    double_mlplr_obj$fit(store_predictions=TRUE)
+    double_mlplr_obj$fit(store_predictions = TRUE)
     theta_obj = double_mlplr_obj$coef
     se_obj = double_mlplr_obj$se
     t_obj = double_mlplr_obj$t_stat
@@ -99,7 +100,7 @@ patrick::with_parameters_test_that("Unit tests for PLR:",
       t_external = dml_plr_obj_external$t_stat
       pval_external = dml_plr_obj_external$pval
       ci_external = dml_plr_obj_external$confint(level = 0.95, joint = FALSE)
-      
+
       expect_identical(double_mlplr_obj$smpls, dml_plr_obj_external$smpls)
       expect_equal(theta_external, theta_obj, tolerance = 1e-8)
       expect_equal(se_external, se_obj, tolerance = 1e-8)
